﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using CodeEffects.Rule.Core;
using CodeEffects.Rule.Common;
using CodeEffects.Rule.Asp;
using CodeEffects.Rule.Demo.Asp.Common.Data;

using CodeEffects.Rule.Models;
using System.Xml;

namespace CodeEffects.Rule.Demo.Asp
{
	public partial class Default : Page
	{
		protected override void OnInit(EventArgs e)
		{
			// Respect parents
			base.OnInit(e);

			// Attach handlers to events of page controls
			this.ddlTheme.SelectedIndexChanged += new EventHandler(this.ThemeChanged);
			this.chkHelp.CheckedChanged += new EventHandler(this.HelpChanged);
			this.chkDots.CheckedChanged += new EventHandler(this.DotsChanged);
			this.btnTest.Click += new EventHandler(this.EvaluateRule);

			// If nested rules are enabled, subscribe to their events
			if (this.ruleControl.ShowToolBar)
			{
				this.ruleControl.SaveRule += new SaveEventHandler(this.SaveRule);
				this.ruleControl.DeleteRule += new RuleEventHandler(this.DeleteRule);
				this.ruleControl.LoadRule += new RuleEventHandler(this.LoadRule);
			}
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			this.ruleControl.GetRuleDelegate = Storage.LoadRuleXml;

			// Load only evaluation rules for context menus. No need to sort them - Web Rule will
			// mix these rules with rule fields and sort the whole list
			this.ruleControl.ContextMenuRules = Storage.GetAllRules();

			// If rule management is enabled, load rules for context menus and toolbar
			if (this.ruleControl.ShowToolBar)
			{
				// Load all your rules into the tool bar's rules menu. You need these rules there
				// in order to be able to load, edit and delete them.
				this.ruleControl.ToolBarRules = this.ruleControl.Mode == RuleType.Execution ? Storage.GetAllRules() : this.ruleControl.ContextMenuRules;
				if (this.ruleControl.ToolBarRules.Count > 0) Storage.SortRules(this.ruleControl.ToolBarRules);
			}

			if (!this.IsPostBack)
			{
				this.chkHelp.Checked = this.ruleControl.ShowHelpString;
				this.chkDots.Checked = this.ruleControl.ShowLineDots;

				// Physician drop down list
				foreach(DataSourceItem item in Physician.List())
				{
					this.ddlPhysician.Items.Add(new ListItem(item.Name, item.ID.ToString()));
					this.ddlPhysician2.Items.Add(new ListItem(item.Name, item.ID.ToString()));
				}

				// Education drop down. We just add the same data to the list as returned by the
				// getEducationTypes() client function (declared in the /Common/Controls.Site.Master file)
				// in order not to create an additional type that would sync the client and server lists
				this.ddlEducation.Items.Add(new ListItem("Master Degree", "1"));
				this.ddlEducation.Items.Add(new ListItem("College", "2"));
				this.ddlEducation.Items.Add(new ListItem("High School", "3"));
				this.ddlEducation.Items.Add(new ListItem("Other", "4"));
			}
		}

		private void LoadRule(object sender, RuleEventArgs e)
		{
			try
			{
				// Load Rule XML from storage as string XML
				string ruleXml = Storage.LoadRuleXml(e);

				// Load that string XML into the rule control
				this.ruleControl.LoadRuleXml(ruleXml);
			}
			catch (RuleException ex)
			{
				this.lblInfo.Text = "Error loading the rule: " + ex.Message;
			}
		}

		private void DeleteRule(object sender, RuleEventArgs e)
		{
			try
			{
				// Delete the file of this rule
				Storage.DeleteRule(e);

				// Remove this rule from the ToolBarRules collection
				Storage.RemoveRuleFromCollection(this.ruleControl.ToolBarRules, e.RuleID);

				// Also, remove this rule from context menu
				if ((bool)e.IsEvaluationTypeRule)
					Storage.RemoveRuleFromCollection(this.ruleControl.ContextMenuRules, e.RuleID);

				// Clear the Rule Area
				this.ruleControl.Clear();

				// Notify the user
				this.lblInfo.Text = "The rule was deleted successfully";
			}
			catch (RuleException ex)
			{
				this.lblInfo.Text = ex.Message;
			}
		}

		private void SaveRule(object sender, SaveEventArgs e)
		{
			// First check if the rule is empty or invalid
			if(this.ruleControl.IsEmpty || !this.ruleControl.IsValid)
			{
				this.lblInfo.Text = "The rule is either empty or invalid.";
				return;
			}

			// Save this rule as XML file
			Storage.SaveRule(e);

			// Add this rule to the Rules menu in the Toolbar in order
			// for rule author to be able to load it for editing
			Storage.AddRuleToCollection(
				this.ruleControl.ToolBarRules, e.RuleID, e.RuleName, e.RuleDescription);
			Storage.SortRules(this.ruleControl.ToolBarRules);

			// If it's an evaluation type rule, add it to the context menu
			// in order for rule author to be able to use it as a reusable rule
			if (e.IsEvaluationTypeRule)
				Storage.AddRuleToCollection(
					this.ruleControl.ContextMenuRules, e.RuleID, e.RuleName, e.RuleDescription);

			// Notify the user
			this.lblInfo.Text = "Rule was saved successfully";
		}

		private void EvaluateRule(object sender, EventArgs e)
		{
			if (this.ruleControl.IsEmpty || !this.ruleControl.IsValid)
			{
				// Only check if Web Rule is empty or contains an invalid rule; control will take care of the rest
				this.lblInfo.Text = "Rule area is empty or the current rule is invalid.";
				return;
			}

			// Get the rule XML
			string rule = this.ruleControl.GetRuleXml();

			// Create new instance of Evaluator, passing it the rule XML
			Evaluator<Patient> evaluator = new Evaluator<Patient>(rule, Storage.LoadRuleXml);

			try
			{
				Patient patient = this.GetPatient();

				// Evaluate the rule against the patient
				bool success = evaluator.Evaluate(patient);

				if(!string.IsNullOrWhiteSpace(patient.Output)) this.lblInfo.Text = patient.Output;
				else this.lblInfo.Text = "The current rule evaluated to " + success;
			}
			catch(FormatException)
			{
				// Controls may throw some formatting exceptions in the GetPatient() method
				this.lblInfo.Text = "Some page controls contain invalid data";
			}
		}

		private Patient GetPatient()
		{
			Patient patient = new Patient();
			patient.ID = Guid.NewGuid();

			// Info
			if (!string.IsNullOrWhiteSpace(this.txtDOB.Text)) patient.DOB = DateTime.Parse(this.txtDOB.Text);
			if (!string.IsNullOrWhiteSpace(this.txtEmail.Text)) patient.Email = this.txtEmail.Text;
			if (!string.IsNullOrWhiteSpace(this.txtFirstName.Text)) patient.FirstName = this.txtFirstName.Text;
			if (!string.IsNullOrWhiteSpace(this.txtLastName.Text)) patient.LastName = this.txtLastName.Text;
			patient.Gender = (Gender)Enum.Parse(typeof(Gender), this.ddlGender.SelectedValue);
			patient.PhysicianID = int.Parse(this.ddlPhysician.SelectedValue);
			patient.EducationTypeID = int.Parse(this.ddlEducation.SelectedValue);

			// Home
			patient.Home = new Address();
			if (!string.IsNullOrWhiteSpace(this.txtHomeCity.Text)) patient.Home.City = this.txtHomeCity.Text;
			if (!string.IsNullOrWhiteSpace(this.txtHomeStreet.Text)) patient.Home.Street = this.txtHomeStreet.Text;
			if (!string.IsNullOrWhiteSpace(this.txtHomeZip.Text)) patient.Home.Zip = this.txtHomeZip.Text;
			patient.Home.State = (State)Enum.Parse(typeof(State), this.ddlHomeState.SelectedValue);

			// Business
			patient.Work = new Address();
			if(!string.IsNullOrWhiteSpace(this.txtWorkCity.Text)) patient.Work.City = this.txtWorkCity.Text;
			if(!string.IsNullOrWhiteSpace(this.txtWorkStreet.Text)) patient.Work.Street = this.txtWorkStreet.Text;
			if(!string.IsNullOrWhiteSpace(this.txtWorkZip.Text)) patient.Work.Zip = this.txtWorkZip.Text;
			patient.Work.State = (State)Enum.Parse(typeof(State), this.ddlWorkState.SelectedValue);

			// Current
			if (!string.IsNullOrWhiteSpace(this.txtPulse.Text)) patient.Pulse = int.Parse(this.txtPulse.Text);
			if (!string.IsNullOrWhiteSpace(this.txtSysPressure.Text)) patient.SystolicPressure = int.Parse(this.txtSysPressure.Text);
			if (!string.IsNullOrWhiteSpace(this.txtDiaPressure.Text)) patient.DiastolicPressure = int.Parse(this.txtDiaPressure.Text);
			if (!string.IsNullOrWhiteSpace(this.txtTemperature.Text)) patient.Temperature = int.Parse(this.txtTemperature.Text);

			// History
			patient.Headaches = this.chkHeadache.Checked;
			patient.Alcohol = this.chkAlcohol.Checked;
			patient.Tobacco = this.chkSmoker.Checked;
			patient.Allergies = this.chkAllergy.Checked;

			return patient;
		}

		private void ThemeChanged(object sender, EventArgs e)
		{
			this.ruleControl.Theme = (ThemeType)Enum.Parse(typeof(ThemeType), this.ddlTheme.SelectedValue);
		}

		private void DotsChanged(object sender, EventArgs e)
		{
			this.ruleControl.ShowLineDots = this.chkDots.Checked;
		}

		private void HelpChanged(object sender, EventArgs e)
		{
			this.ruleControl.ShowHelpString = this.chkHelp.Checked;
		}
	}
}